<?php

namespace App\Controller\Admin\Device;

use App\Bundles\Sg\DatatablesBundle\Response\DatatableQueryBuilder;
use App\Bundles\Sg\DatatablesBundle\Response\DatatableResponse;
use App\Controller\Admin\BaseController;
use App\Entity\AgentProfile\AgentProfile;
use App\Entity\Alert\Alert;
use App\Entity\ConfigProfile;
use App\Entity\Device;
use App\Entity\DeviceOsVersion;
use App\Entity\DeviceType;
use App\Entity\Label;
use App\Entity\LabelDeviceMap;
use App\Entity\MonitoringGroup;
use App\Entity\MonitoringGroupMapping;
use App\Exception\AlertNotFoundException;
use App\Exception\DeviceNotFoundException;
use App\Exception\DeviceNumberInvalidException;
use App\Exception\MonitoringGroupNotFoundException;
use App\Form\DeleteType;
use App\Form\Device\ConfigProfileCloneDeviceToType;
use App\Form\Device\ConfigProfileCloneFromDeviceType;
use App\Form\Device\DeviceAgentProfileType;
use App\Form\Device\DeviceCustomModuleDownloadType;
use App\Form\Device\DeviceHostnameType;
use App\Form\Device\DeviceLabelType;
use App\Form\Device\FwVersionType;
use App\Form\Device\MonitoringGroupType;
use App\Repository\ConfigProfileRepository;
use App\Repository\DeviceRepository;
use App\Repository\DeviceUserModuleRepository;
use App\Repository\MonitoringGroupRepository;
use App\Repository\ProductFlagsRepository;
use App\Service\Api\AgentProfiles;
use App\Service\Api\ApiService;
use App\Service\Api\DashboardLights;
use App\Service\Command;
use App\Service\DatatableService;
use App\Service\History;
use Doctrine\DBAL\Exception;
use Doctrine\ORM\EntityManagerInterface;
use Doctrine\Persistence\ManagerRegistry;
use Sg\DatatablesBundle\Datatable\DatatableFactory;
use Symfony\Component\DependencyInjection\ParameterBag\ParameterBagInterface;
use Symfony\Component\HttpFoundation\BinaryFileResponse;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\ResponseHeaderBag;
use Symfony\Component\Routing\Annotation\Route;

class DeviceController extends BaseController
{

    /**
     * @var EntityManagerInterface
     */
    private EntityManagerInterface $entityManager;

    /**
     * @var DeviceRepository
     */
    private DeviceRepository $deviceRepository;

    /**
     * @var ProductFlagsRepository
     */
    private ProductFlagsRepository $productFlagsRepository;
    private $render;

    /**
     * @param ManagerRegistry $managerRegistry
     * @param EntityManagerInterface $entityManager
     * @param DeviceRepository $deviceRepository
     * @param ProductFlagsRepository $productFlagsRepository
     */
    public function __construct(ManagerRegistry  $managerRegistry, EntityManagerInterface $entityManager,
                                DeviceRepository $deviceRepository, ProductFlagsRepository $productFlagsRepository) {

        parent::__construct($managerRegistry);

        $this->entityManager = $entityManager;
        $this->deviceRepository = $deviceRepository;
        $this->productFlagsRepository = $productFlagsRepository;

    }

    /**
     * @Route("admin/devices", name="admin_devices")
     * @param Request $request
     * @param DatatableResponse $responseService
     * @param DatatableService $datatableService
     * @return JsonResponse|Response
     * @throws \Exception
     */
    public function listDevices(Request $request, DatatableResponse $responseService, DatatableService $datatableService){

        $isAjax = $request->isXmlHttpRequest();
        $this->filter = $request->query->get('column');

        $datatable = $datatableService->getDeviceDatatable();

        if ($isAjax) {
            $responseService->setDatatable($datatable);
            //$responseService->getDatatableQueryBuilder();

            //$datatableQueryBuilder = $responseService->getDatatableQueryBuilder();

            $requestParams = $this->getDatatableFilter($request, $datatable);
            $datatableQueryBuilder = new DatatableQueryBuilder($requestParams, $datatable);
            $responseService->setDatatableQueryBuilder($datatableQueryBuilder);

            $qb = $datatableQueryBuilder->getQb();
            $qb->andWhere('vdevice.deleted = :deleted');
            $qb->setParameter('deleted', '0');
            
            return $responseService->getResponse();
        }

        return $this->render('device/index.html.twig', array(
            'datatable' => $datatable,
            'filter' => $this->filter,
            'title' => 'Registered Devices'
        ));

    }

    /**
     * @Route("admin/devices/alert/{alertId}", name="admin_devices_triggered_alert")
     * @param int $alertId
     * @param Request $request
     * @param ManagerRegistry $managerRegistry
     * @param ApiService $apiService
     * @param DatatableResponse $responseService
     * @param DatatableService $datatableService
     * @return JsonResponse|Response
     * @throws AlertNotFoundException
     */
    public function listDevicesByAlertId(int $alertId, Request $request, ManagerRegistry $managerRegistry, ApiService $apiService,
                                         DatatableResponse $responseService, DatatableService $datatableService){

        $isAjax = $request->isXmlHttpRequest();
        $this->filter = $request->query->get('column');

        /** @var Alert $alert */
        $alert = $managerRegistry->getRepository(Alert::class)->findOneBy(['id' => $alertId]);

        if(!$alert){

            throw new AlertNotFoundException();

        }

        $trigerredAlertDevices = $apiService->getTrigerredAlertDevices($alertId);

        $datatable = $datatableService->getDeviceDatatable();

        if ($isAjax) {
            $responseService->setDatatable($datatable);
            $requestParams = $this->getDatatableFilter($request, $datatable);
            $datatableQueryBuilder = new DatatableQueryBuilder($requestParams, $datatable);
            $responseService->setDatatableQueryBuilder($datatableQueryBuilder);

            /** @var QueryBuilder $qb */
            $qb = $datatableQueryBuilder->getQb();
            $qb->andWhere('vdevice.id IN (:deviceIds)');
            $qb->andWhere('vdevice.deleted = :deleted');
            $qb->setParameter('deviceIds', $trigerredAlertDevices);
            $qb->setParameter('deleted', '0');

            return $responseService->getResponse();
        }

        $alertUrl = $this->generateUrl('admin_alert_edit', [
            'alertId' => $alert->getId(),
        ]);

        return $this->render('device/index.html.twig', array(
            'datatable' => $datatable,
            'filter' => $this->filter,
            'title' => 'List of devices for trigerred alert: <a href="' . $alertUrl . '">' . $alert->getName() . '</a>'
        ));

    }

    /**
     * @Route("admin/devices/dashboard/monitoring-group/{monitoringGroupId}/subset/{subsetId}", name="admin_devices_dashboard_light")
     * @param int $monitoringGroupId
     * @param int $subsetId
     * @param Request $request
     * @param ManagerRegistry $managerRegistry
     * @param DashboardLights $dashboardLightsApi
     * @param DatatableResponse $responseService
     * @param DatatableService $datatableService
     * @return JsonResponse|Response
     * @throws MonitoringGroupNotFoundException
     */
    public function listDevicesByDashboardLight(int $monitoringGroupId, int $subsetId, Request $request,
                                                ManagerRegistry $managerRegistry, DashboardLights $dashboardLightsApi,
                                         DatatableResponse $responseService, DatatableService $datatableService){

        $isAjax = $request->isXmlHttpRequest();
        $this->filter = $request->query->get('column');

        /** @var MonitoringGroup $monitoringGroup */
        $monitoringGroup = $managerRegistry->getRepository(MonitoringGroup::class)->findOneBy(['id' => $monitoringGroupId]);

        if(!$monitoringGroup){

            throw new MonitoringGroupNotFoundException();

        }

        $triggeredDevices = $dashboardLightsApi->getDashboardDetailData($monitoringGroup->getId(), $subsetId);

        $datatable = $datatableService->getDeviceDatatable();

        if ($isAjax) {
            $responseService->setDatatable($datatable);
            $requestParams = $this->getDatatableFilter($request, $datatable);
            $datatableQueryBuilder = new DatatableQueryBuilder($requestParams, $datatable);
            $responseService->setDatatableQueryBuilder($datatableQueryBuilder);

            /** @var QueryBuilder $qb */
            $qb = $datatableQueryBuilder->getQb();
            $qb->andWhere('vdevice.id IN (:deviceIds)');
            $qb->andWhere('vdevice.deleted = :deleted');
            $qb->setParameter('deviceIds', $triggeredDevices);
            $qb->setParameter('deleted', '0');

            return $responseService->getResponse();
        }

        if($subsetId === 0){
            $dashboardDeviceStatusType = 'Online';
        }elseif ($subsetId === 1){
            $dashboardDeviceStatusType = 'Delayed';
        }elseif ($subsetId === 2){
            $dashboardDeviceStatusType = 'Late';
        }elseif ($subsetId === 3){
            $dashboardDeviceStatusType = 'Synced';
        }elseif ($subsetId === 4){
            $dashboardDeviceStatusType = 'Pending';
        }elseif ($subsetId === 5){
            $dashboardDeviceStatusType = 'Error';
        }else{
            $dashboardDeviceStatusType = '';
        }

        return $this->render('device/index.html.twig', array(
            'datatable' => $datatable,
            'filter' => $this->filter,
            //'title' => 'List of devices for trigerred alert: <a href="' . $alertUrl . '">' . $alert->getName() . '</a>'
            'title' => 'List of ' . $dashboardDeviceStatusType .' devices:'
        ));

    }

    /**
     * @Route("admin/devices/past-alert/{alertId}", name="admin_devices_past_alert")
     * @param int $alertId
     * @param Request $request
     * @param ManagerRegistry $managerRegistry
     * @param ApiService $apiService
     * @param DatatableResponse $responseService
     * @param DatatableService $datatableService
     * @return JsonResponse|Response
     * @throws AlertNotFoundException
     */
    public function listDevicesByPastAlertId(int $alertId, Request $request, ManagerRegistry $managerRegistry, ApiService $apiService,
                                         DatatableResponse $responseService, DatatableService $datatableService){

        $isAjax = $request->isXmlHttpRequest();
        $this->filter = $request->query->get('column');

        /** @var Alert $alert */
        $alert = $managerRegistry->getRepository(Alert::class)->findOneBy(['id' => $alertId]);

        if(!$alert){

            throw new AlertNotFoundException();

        }

        $pastAlertDevices = $apiService->getPastAlertDevices($alertId);

        $datatable = $datatableService->getDeviceDatatable();

        if ($isAjax) {
            $responseService->setDatatable($datatable);
            $requestParams = $this->getDatatableFilter($request, $datatable);
            $datatableQueryBuilder = new DatatableQueryBuilder($requestParams, $datatable);
            $responseService->setDatatableQueryBuilder($datatableQueryBuilder);

            /** @var QueryBuilder $qb */
            $qb = $datatableQueryBuilder->getQb();
            $qb->andWhere('vdevice.id IN (:deviceIds)');
            $qb->andWhere('vdevice.deleted = :deleted');
            $qb->setParameter('deviceIds', $pastAlertDevices);
            $qb->setParameter('deleted', '0');

            return $responseService->getResponse();
        }

        $alertUrl = $this->generateUrl('admin_alert_edit', [
            'alertId' => $alert->getId(),
        ]);

        $this->render = $this->render('device/index.html.twig', array(
            'datatable' => $datatable,
            'filter' => $this->filter,
            'title' => 'List of devices for past alert: <a href="' . $alertUrl . '">' . $alert->getName() . '</a>'
        ));
        return $this->render;

    }

    /**
     * @Route("admin/device/{deviceId}", name="device_detail")
     * @param $deviceId
     * @param ManagerRegistry $managerRegistry
     * @param DeviceRepository $deviceRepository
     * @param ConfigProfileRepository $configProfileRepository
     * @param History $historyService
     * @param Command $commandService
     * @param ParameterBagInterface $params
     * @param ApiService $apiService
     * @param Request $request
     * @param DeviceUserModuleRepository $userModuleRepository
     * @return RedirectResponse|Response|void
     * @throws DeviceNotFoundException
     * @throws DeviceNumberInvalidException
     * @throws Exception
     */
    public function deviceDetail($deviceId, ManagerRegistry $managerRegistry, DeviceRepository $deviceRepository,
                                 ConfigProfileRepository $configProfileRepository, History $historyService, Command $commandService,
                                 ParameterBagInterface $params, ApiService $apiService, Request $request,
                                 DeviceUserModuleRepository $userModuleRepository, MonitoringGroupRepository $monitoringGroupRepository,
                                 AgentProfiles $agentProfilesApi)
    {

        if (!is_numeric($deviceId)) {

            throw new DeviceNumberInvalidException();

        }

        /** @var Device $device */
        $device = $managerRegistry->getRepository(Device::class)->findOneBy(['id' => $deviceId]);

        if (!$device || $device->isDeleted()) {

            throw new DeviceNotFoundException();

        }

        $findBy = ['available' => 1, 'approved' => 1];

        if (!is_null($device->getOsVer())) {

            $findBy['osType'] = $device->getOsVer()->getOsType();

        }

        $osVersions = $managerRegistry->getRepository(DeviceOsVersion::class)->findBy($findBy, ['version' => 'DESC']);

        $clonedFromDeviceId = (!empty($device->getConfigProfile())) ? $device->getConfigProfile()->getAutomatic()->getId() : null;
        $devicesToCloneToExcludeDeviceIds = [$device->getId()];
        if (isset($clonedFromDeviceId)) {
            $devicesToCloneToExcludeDeviceIds[] = $clonedFromDeviceId;
        }

        $devicesTocloneTo = $deviceRepository->getDevicesExcept($devicesToCloneToExcludeDeviceIds, $device->getDevType()->getId());

        $deviceToClonedToResult = $configProfileRepository->findOneBy(['automatic' => $device]);
        $devicesToClonedTo = [];
        if ($deviceToClonedToResult) {
            $devicesToClonedTo = $deviceRepository->findBy(['configProfile' => $deviceToClonedToResult]);

            foreach ($devicesToClonedTo as $deviceToClonedTo) {

                $devicesToClonedToIds[] = $deviceToClonedTo->getId();

            }
        }

        $devicesToCloneFromExcludeDeviceIds[] = $device->getId();
        if (isset($devicesToClonedToIds)) {
            $devicesToCloneFromExcludeDeviceIds = array_merge($devicesToCloneFromExcludeDeviceIds, $devicesToClonedToIds);
        }

        $devicesToCloneFrom = $configProfileRepository->getDevicesToBeClonedFromPlainSql($devicesToCloneFromExcludeDeviceIds, $device->getDevType()->getId());

        $fwVersionForm = $this->createForm(FwVersionType::class, ['osVersions' => $osVersions, 'showNoSpecific' => !is_null($device->getOsVerDesired()), 'device' => $device]);
        $deleteForm = $this->createForm(DeleteType::class, ['text' => 'Delete device']);
        $configProfileCloneFromForm = $this->createForm(ConfigProfileCloneFromDeviceType::class, ['device' => $device, 'devicesToCloneFrom' => $devicesToCloneFrom, 'isCloned' => !is_null($device->configProfile)]);
        $configProfileCloneToForm = $this->createForm(ConfigProfileCloneDeviceToType::class, ['devicesToClonedTo' => $devicesToClonedTo, 'devicesToCloneTo' => $devicesTocloneTo, 'isCloned' => (count($devicesToClonedTo) > 0)]);
        $hostnameForm = $this->createForm(DeviceHostnameType::class, ['device' => $device]);
        $customModuleDownloadForm = $this->createForm(DeviceCustomModuleDownloadType::class, ['device' => $device]);

        $labels = $managerRegistry->getRepository(Label::class)->findAll();
        $deviceLabels = $device->labels;
        $labelsForm = $this->createForm(DeviceLabelType::class, ['deviceLabels' => $deviceLabels, 'labels' => $labels]);

        $monitoringGroups = $monitoringGroupRepository->getAllMonitoringGroups();
        $deviceMonitoringGroups = $device->monitoringGroupMappings;
        $monitoringGroupsForm = $this->createForm(MonitoringGroupType::class, ['deviceMonitoringGroups' => $deviceMonitoringGroups, 'monitoringGroups' => $monitoringGroups]);

        $agentProfiles = $managerRegistry->getRepository(AgentProfile::class)->findAll();
        $agentProfilesForm = $this->createForm(DeviceAgentProfileType::class, ['device' => $device, 'agentProfiles' => $agentProfiles]);

        $packages = $userModuleRepository->getDeviceUserModules($device);

        $fwVersionForm->handleRequest($request);
        $deleteForm->handleRequest($request);
        $configProfileCloneFromForm->handleRequest($request);
        $configProfileCloneToForm->handleRequest($request);
        $hostnameForm->handleRequest($request);
        $labelsForm->handleRequest($request);
        $monitoringGroupsForm->handleRequest($request);
        $agentProfilesForm->handleRequest($request);
        $customModuleDownloadForm->handleRequest($request);

        $historyTypeId = 1;
        $historyTimeLegend = '"' . implode('","', $historyService->getHistoryTimeLegend($historyTypeId)) . '"';
        $historyData = $historyService->getHistoryDataForDevice($historyTypeId, $device->getId());

        /*var_dump($historyData);
        die();*/

        if ($deleteForm->isSubmitted() && $deleteForm->isValid()) {

            if ($apiService->deleteDevice($device->getId()) !== false) {

                $this->addFlash(
                    'success',
                    'Device was successfully deleted.'
                );

            } else {

                $this->addFlash(
                    'danger',
                    'Device was not deleted.'
                );

            }

            /*try{

                $deviceRepository->deleteDevice($device->getId());

                $this->addFlash(
                    'success',
                    'Device was successfully deleted.'
                );

            }catch (\Exception $e){

                $this->addFlash(
                    'danger',
                    'Device was not deleted.'
                );

            }*/

            return $this->redirectToRoute('admin_devices');

        }

        if ($customModuleDownloadForm->isSubmitted() && $customModuleDownloadForm->isValid()) {

            $formData = $customModuleDownloadForm->getData();

            $devId = $formData['deviceId'];
            $deviceTypeId = $formData['deviceType'];

            $exitCode = $commandService->runCustomizedAgentGenerator($devId);

            if ($exitCode === 0) {

                $deviceType = $managerRegistry->getRepository(DeviceType::class)->findOneBy(['id' => $deviceTypeId]);

                $mapping = $deviceType->getDeviceTypeAgentMapping();
                if (count($mapping) > 1) {

                    $this->addFlash(
                        'danger',
                        'No valid agent data given.'
                    );

                } else {

                    $customizedAgentPath = $params->get('app.customizedAgentPath');
                    $agent = $mapping[0]->getDeviceAgent();

                    $file = $customizedAgentPath . $devId . DIRECTORY_SEPARATOR . $agent->getDirName() . DIRECTORY_SEPARATOR . $agent->getPackageName();

                    $response = new BinaryFileResponse($file);
                    $response->setContentDisposition(
                        ResponseHeaderBag::DISPOSITION_ATTACHMENT,
                        $agent->getPackageName()
                    );

                    return $response;
                }

            } else {

                $this->addFlash(
                    'danger',
                    'The agent cannot be created.'
                );

            }

            return $this->redirectToRoute('device_detail', ['deviceId' => $deviceId]);

        }

        if ($hostnameForm->isSubmitted() && $hostnameForm->isValid()) {

            // $form->getData() holds the submitted values
            // but, the original `$task` variable has also been updated
            $formData = $hostnameForm->getData();

            $device->setHostnameDesired($formData['hostname']);
            $managerRegistry->getManager()->persist($device);
            $managerRegistry->getManager()->flush();

            $this->addFlash(
                'success',
                'Hostname will be updated.'
            );

            return $this->redirectToRoute('device_detail', ['deviceId' => $deviceId]);

        }

        if ($labelsForm->isSubmitted() && $labelsForm->isValid()) {

            $formData = $labelsForm->getData();
            $deviceLabels = $formData['deviceLabels'];
            $labels = $formData['labels'];

            $deviceLabelsIds = [];

            foreach ($deviceLabels as $label) {

                $deviceLabelsIds[] = $label->getLabel()->getId();

            }

            foreach ($labels as $label) {

                //if not exists
                if (!in_array($label, $deviceLabelsIds)) {

                    $label = $managerRegistry->getRepository(Label::class)->findOneBy(['id' => $label]);

                    $labelDeviceMapping = new LabelDeviceMap();
                    $labelDeviceMapping->setLabel($label);
                    $labelDeviceMapping->setDevice($device);
                    $managerRegistry->getManager()->persist($labelDeviceMapping);
                    $managerRegistry->getManager()->flush();

                }

                //Remove processed labels from array
                $key = array_search($label, $deviceLabelsIds);
                if (false !== $key) {
                    unset($deviceLabelsIds[$key]);
                }

            }

            foreach ($deviceLabelsIds as $labelDeleteId) {

                $label = $managerRegistry->getRepository(Label::class)->findOneBy(['id' => $labelDeleteId]);
                $device = $managerRegistry->getRepository(Device::class)->findOneBy(['id' => $device->getId()]);
                $labelDeviceMapping = $managerRegistry->getRepository(LabelDeviceMap::class)->findOneBy(
                    ['label' => $label, 'device' => $device]);

                if ($labelDeviceMapping) {

                    $managerRegistry->getManager()->remove($labelDeviceMapping);
                    $managerRegistry->getManager()->flush();

                }

            }

            $this->addFlash(
                'success',
                'Labels successfully saved.'
            );

            return $this->redirectToRoute('device_detail', ['deviceId' => $deviceId]);

        }

        if ($monitoringGroupsForm->isSubmitted() && $monitoringGroupsForm->isValid()) {

            $formData = $monitoringGroupsForm->getData();
            $deviceMonitoringGroups = $formData['deviceMonitoringGroups'];
            $monitoringGroups = $formData['monitoringGroups'];

            $deviceMonitoringGroupsIds = [];

            foreach ($deviceMonitoringGroups as $monitoringGroup) {

                if($monitoringGroup->getMonitoringGroup()->getId() === 1)
                    continue;
                $deviceMonitoringGroupsIds[] = $monitoringGroup->getMonitoringGroup()->getId();

            }

            foreach ($monitoringGroups as $monitoringGroup) {

                //if not exists
                if (!in_array($monitoringGroup, $deviceMonitoringGroupsIds)) {

                    $monitoringGroup = $managerRegistry->getRepository(MonitoringGroup::class)->findOneBy(['id' => $monitoringGroup]);

                    $monitoringGroupDeviceMapping = new MonitoringGroupMapping();
                    $monitoringGroupDeviceMapping->setMonitoringGroup($monitoringGroup);
                    $monitoringGroupDeviceMapping->setDevice($device);
                    $managerRegistry->getManager()->persist($monitoringGroupDeviceMapping);
                    $managerRegistry->getManager()->flush();

                }

                //Remove processed labels from array
                $key = array_search($monitoringGroup, $deviceMonitoringGroupsIds);
                if (false !== $key) {
                    unset($deviceMonitoringGroupsIds[$key]);
                }

            }

            foreach ($deviceMonitoringGroupsIds as $monitoringGroupDeleteId) {

                $monitoringGroup = $managerRegistry->getRepository(MonitoringGroup::class)->findOneBy(['id' => $monitoringGroupDeleteId]);
                $device = $managerRegistry->getRepository(Device::class)->findOneBy(['id' => $device->getId()]);
                $monitoringGroupDeviceMapping = $managerRegistry->getRepository(MonitoringGroupMapping::class)->findOneBy(
                    ['monitoringGroup' => $monitoringGroup, 'device' => $device]);

                if ($monitoringGroupDeviceMapping) {

                    $managerRegistry->getManager()->remove($monitoringGroupDeviceMapping);
                    $managerRegistry->getManager()->flush();

                }

            }

            $this->addFlash(
                'success',
                'Monitoring groups successfully saved.'
            );

            return $this->redirectToRoute('device_detail', ['deviceId' => $deviceId]);

        }

        if ($fwVersionForm->isSubmitted() && $fwVersionForm->isValid()){

            // $form->getData() holds the submitted values
            // but, the original `$task` variable has also been updated
            $deviceFwForm = $fwVersionForm->getData();

            if($fwVersionForm->offsetExists('no_specific') && $fwVersionForm->get('no_specific')->isClicked()){

                $device->setOsVerDesired(null);
                $managerRegistry->getManager()->persist($device);
                $managerRegistry->getManager()->flush();

                $this->addFlash(
                    'success',
                    'Fw version was set to no specific.'
                );

                return $this->redirectToRoute('device_detail', ['deviceId' => $deviceId]);

            }else{

                //if($device->getOsVer()->getId() !== $deviceFwForm['fw_version']){

                    $osVersion = $managerRegistry->getRepository(DeviceOsVersion::class)
                        ->findOneBy(['id' =>  $deviceFwForm['fw_version']]);
                    //dump($osVersion);
                    //die();

                    $device->setOsVerDesired($osVersion);
                    $managerRegistry->getManager()->persist($device);
                    $managerRegistry->getManager()->flush();

                    $this->addFlash(
                        'success',
                        'New Fw version was set.'
                    );

                    return $this->redirectToRoute('device_detail', ['deviceId' => $deviceId]);

                //}

            }

            return $this->redirectToRoute('device_detail', ['deviceId' => $deviceId]);
        }

        if ($agentProfilesForm->isSubmitted() && $agentProfilesForm->isValid()){

            $formData = $agentProfilesForm->getData();
            $agentProfileId = $formData['agent_profile'];

            $agentProfile = $managerRegistry->getRepository(AgentProfile::class)
                ->findOneBy(['id' =>  $agentProfileId]);

            $result = $agentProfilesApi->assignDeviceAgentProfile($agentProfileId, [$deviceId]);

            /*$device->setAgentProfile($agentProfile);
            $managerRegistry->getManager()->persist($device);
            $managerRegistry->getManager()->flush();*/

            $this->addFlash(
                'success',
                'Agent profile was updated.'
            );

            return $this->redirectToRoute('device_detail', ['deviceId' => $deviceId]);

        }

        if ($configProfileCloneFromForm->isSubmitted() && $configProfileCloneFromForm->isValid()) {

            $formData = $configProfileCloneFromForm->getData();

            if($configProfileCloneFromForm->offsetExists('no_clone') && $configProfileCloneFromForm->get('no_clone')->isClicked()){

                $device->setConfigProfile(null);
                $managerRegistry->getManager()->persist($device);
                $managerRegistry->getManager()->flush();

                $this->addFlash(
                    'success',
                    'Device cloning was removed.'
                );


            }else{

                $configProfileId = $formData['config_profile'];

                $configProfile = $managerRegistry->getRepository(ConfigProfile::class)
                    ->findOneBy(['id' =>  $configProfileId]);

                $device->setConfigProfile($configProfile);
                $managerRegistry->getManager()->persist($device);
                $managerRegistry->getManager()->flush();

                $this->addFlash(
                    'success',
                    'Device config will be cloned.'
                );

            }

            return $this->redirectToRoute('device_detail', ['deviceId' => $deviceId]);

        }

        if ($configProfileCloneToForm->isSubmitted() && $configProfileCloneToForm->isValid()) {

            if ($configProfileCloneToForm->offsetExists('no_clone') && $configProfileCloneToForm->get('no_clone')->isClicked()) {

                $devicesToClonedToIds = [];
                $devicesToDelete = [];

                foreach ($devicesToClonedTo as $device) {

                    $devicesToClonedToIds[] = $device->getId();

                }

                foreach ($devicesToClonedToIds as $deviceToDeleteId) {

                    $devicesToDelete[] = $deviceToDeleteId;

                }

                if(!empty($devicesToDelete)){

                    $result = $this->updateDevicesConfigProfile(null, $devicesToDelete);

                }

                if($result){

                    $this->addFlash(
                        'success',
                        'Device cloning was removed.'
                    );

                }else{

                    $this->addFlash(
                        'danger',
                        'Device cloning remove was unsuccessful.'
                    );

                }

                return $this->redirectToRoute('device_detail', ['deviceId' => $deviceId]);


            } else {

                $formData = $configProfileCloneToForm->getData();
                $devicesToCloneTo = $formData['devices'];
                $configProfileToClone = $device->configProfileAutomatic;

                if (!$configProfileToClone) {

                    $this->addFlash(
                        'danger',
                        'Device profile is empty and cannot be cloned.'
                    );

                    return $this->redirectToRoute('device_detail', ['deviceId' => $deviceId]);

                }

                $devicesToClonedToIds = [];

                foreach ($devicesToClonedTo as $device) {

                    $devicesToClonedToIds[] = $device->getId();

                }

                $devicesCloneTo = [];

                foreach ($devicesToCloneTo as $deviceCloneToId) {

                    //if not exists
                    if (!in_array($deviceCloneToId, $devicesToClonedToIds)) {

                        $devicesCloneTo[] = $deviceCloneToId;

                    }

                    //Remove processed devices from array
                    $key = array_search($deviceCloneToId, $devicesToClonedToIds);
                    if (false !== $key) {
                        unset($devicesToClonedToIds[$key]);
                    }

                }

                if(!empty($devicesCloneTo)){

                    $result = $this->updateDevicesConfigProfile($configProfileToClone->getId(), $devicesCloneTo);

                }

                $devicesToDelete = [];

                foreach ($devicesToClonedToIds as $deviceToDeleteId) {

                    $devicesToDelete[] = $deviceToDeleteId;

                }

                if(!empty($devicesToDelete)){

                    $result = $this->updateDevicesConfigProfile(null, $devicesToDelete);

                }

                if(!isset($result)){

                    $this->addFlash(
                        'success',
                        'No change processed for devices to be cloned to.'
                    );

                }else{

                    if($result){

                        $this->addFlash(
                            'success',
                            'Device profile was cloned to selected devices.'
                        );

                    }else{

                        $this->addFlash(
                            'danger',
                            'Device profile cloned to selected failed.'
                        );

                    }

                }

                return $this->redirectToRoute('device_detail', ['deviceId' => $deviceId]);

            }

        }

        return $this->render('device/detail.html.twig', array(
            'device' => $device,
            'hostnameForm' => $hostnameForm->createView(),
            'fwForm' => $fwVersionForm->createView(),
            'deleteForm' => $deleteForm->createView(),
            'isDevicesToCloneFromEmpty' => empty($devicesToCloneFrom),
            'configProfileCloneFromForm' => $configProfileCloneFromForm->createView(),
            'configProfileCloneToForm' => $configProfileCloneToForm->createView(),
            'customModuleDownloadForm' => $customModuleDownloadForm->createView(),
            'labelsForm' => $labelsForm->createView(),
            'monitoringGroupsForm' => $monitoringGroupsForm->createView(),
            'agentProfilesForm' => $agentProfilesForm->createView(),
            'packages' => $packages,
            'historyTimeLegend' => $historyTimeLegend,
            'osVersions' => $osVersions,
            'historyData' => json_encode($historyData/*, JSON_PRETTY_PRINT | ENT_QUOTES*/),
        ));

    }

    /**
     * @param int|null $configProfileId
     * @param array $devicesId
     * @return bool
     * @throws Exception
     */
    private function updateDevicesConfigProfile(?int $configProfileId, array $devicesId): bool {

        try{

            $this->entityManager->getConnection()->beginTransaction();
            $this->deviceRepository->updateDevicesConfigProfile($configProfileId, $devicesId);
            $this->productFlagsRepository->incrementDevchgVersion();
            $this->entityManager->getConnection()->commit();

            return true;

        }catch (\Exception $e){

            $this->entityManager->getConnection()->rollBack();
            return false;

        }

    }

}