<?php

declare(strict_types=1);

namespace App\Entity\Alert;

use App\Entity\Device;
use App\Entity\MonitoringGroup;
use App\Entity\TypesOperator;
use Doctrine\ORM\Mapping as ORM;

/**
 * @ORM\Entity
 * @ORM\Table(name="alerts")
 */
class Alert
{
    /**
     * @var int|null
     * @ORM\Column(name="id", type="integer")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    private ?int $id = null;

    /**
     * @var string|null
     * @ORM\Column(type="string", length=64, nullable=true)
     */
    public ?string $name = null;

    /**
     * @var AlertTargetType|null
     * @ORM\ManyToOne(targetEntity="App\Entity\Alert\AlertTargetType", inversedBy="alerts")
     * @ORM\JoinColumn(name="target_type")
     */
    public ?AlertTargetType $targetType = null;

    /**
     * @var AlertTargetSpec|null
     * @ORM\ManyToOne(targetEntity="App\Entity\Alert\AlertTargetSpec", inversedBy="alerts")
     * @ORM\JoinColumn(name="target_spec")
     */
    public ?AlertTargetSpec $targetSpec = null;

    /**
     * @var int|null
     * @ORM\Column(type="smallint", nullable=true, options={"default" : 1})
     */
    public ?int $targetAmount = null;

    /**
     * @var Device|null
     * @ORM\ManyToOne(targetEntity="App\Entity\Device", inversedBy="alerts")
     * @ORM\JoinColumn(name="target_device_id", referencedColumnName="id_device")
     */
    public ?Device $targetDevice = null;

    /**
     * @var MonitoringGroup|null
     * @ORM\ManyToOne(targetEntity="App\Entity\MonitoringGroup", inversedBy="alerts")
     * @ORM\JoinColumn(name="target_mongrp_id", referencedColumnName="id")
     */
    public ?MonitoringGroup $targetMonitoringGroup = null;

    /**
     * @var int
     * @ORM\Column(type="integer", nullable=true, options={"default": 120})
     */
    public int $evaluateInterval;

    /**
     * @var TypesOperator|null
     * @ORM\ManyToOne(targetEntity="App\Entity\TypesOperator", inversedBy="alertsOperFirst")
     * @ORM\JoinColumn(name="oper_first")
     */
    public ?TypesOperator $operFirst = null;

    /**
     * @var TypesOperator|null
     * @ORM\ManyToOne(targetEntity="App\Entity\TypesOperator", inversedBy="alertsOperSecond")
     * @ORM\JoinColumn(name="oper_second")
     */
    public ?TypesOperator $operSecond = null;

    /**
     * @var float|null
     * @ORM\Column(name="num_first", type="float", nullable=true)
     */
    public ?float $numFirst = null;

    /**
     * @var float|null
     * @ORM\Column(name="num_second", type="float", nullable=true)
     */
    public ?float $numSecond = null;

    /**
     * @var \DateTime
     * @ORM\Column(name="last_trigger_time ", type="datetime", options={"default": NULL})
     */
    public ?\DateTime $lastTriggerTime;

    /**
     * @var \DateTime
     * @ORM\Column(name="last_notification_time ", type="datetime", options={"default": NULL})
     */
    public ?\DateTime $lastNotificationTime;

    /**
     * @var bool
     * @ORM\Column(type="boolean", nullable=false, options={"default": false})
     */
    public bool $enabled;

    /**
     * @var bool
     * @ORM\Column(type="boolean", nullable=false, options={"default": false})
     */
    public bool $active;

    /**
     * @var int
     * @ORM\Column(type="smallint", nullable=true, options={"default": "0"})
     */
    public int $counter;

    /**
     * @var int
     * @ORM\Column(type="smallint", nullable=true, options={"default": "0"})
     */
    public int $triggered;

    /**
     * @var bool
     * @ORM\Column(type="boolean", nullable=false, options={"default": false})
     */
    public bool $deleted;

    /**
     * @var int
     * @ORM\Column(type="integer", nullable=true)
     */
    public int $version;

    /**
     * Alert constructor
     */
    public function __construct(){

        $this->enabled = false;
        $this->active = false;
        $this->counter = 0;
        $this->triggered = 0;
        $this->evaluateInterval = 120;
        $this->deleted = false;

    }

    /**
     * @return int|null
     */
    public function getId(): ?int {

        return $this->id;

    }

    /**
     * @return string
     */
    public function getName(): ?string {

        return $this->name;

    }

    /**
     * @return AlertTargetType|null
     */
    public function getTargetType(): ?AlertTargetType {

        return $this->targetType;

    }

    /**
     * @param AlertTargetType $targetType
     */
    public function setTargetType(AlertTargetType $targetType): void {

        $this->targetType = $targetType;

    }

    /**
     * @return AlertTargetSpec|null
     */
    public function getTargetSpec(): ?AlertTargetSpec {

        return $this->targetSpec;

    }

    /**
     * @param AlertTargetSpec $targetSpec
     */
    public function setTargetSpec(AlertTargetSpec $targetSpec): void {

        $this->targetSpec = $targetSpec;

    }

    /**
     * @return int|null
     */
    public function getTargetAmount(): ?int {

        return $this->targetAmount;

    }

    /**
     * @param int $amount
     */
    public function setTargetAmount(int $amount): void {

        $this->targetAmount = $amount;

    }

    /**
     * @return Device|null
     */
    public function getTargetDevice(): ?Device {

        return $this->targetDevice;

    }

    /**
     * @param Device|null $device
     */
    public function setTargetDevice(?Device $device): void {

        $this->targetDevice = $device;

    }

    /**
     * @return MonitoringGroup|null
     */
    public function getTargetMonitoringGroup(): ?MonitoringGroup {

        return $this->targetMonitoringGroup;

    }

    /**
     * @param MonitoringGroup|null $monitoringGroup
     */
    public function setTargetMonitoringGroup(?MonitoringGroup $monitoringGroup): void {

        $this->targetMonitoringGroup = $monitoringGroup;

    }

    /**
     * @return TypesOperator|null
     */
    public function getOperFirst(): ?TypesOperator {

        return $this->operFirst;

    }

    /**
     * @param TypesOperator $operatorType
     */
    public function setOperFirst(TypesOperator $operatorType): void {

        $this->operFirst = $operatorType;

    }

    /**
     * @return float|null
     */
    public function getNumFirst(): ?float {

        return $this->numFirst;

    }

    /**
     * @param float $number
     */
    public function setNumFirst(float $number): void {

        $this->numFirst = $number;

    }

    /**
     * @return TypesOperator|null
     */
    public function getOperSecond(): ?TypesOperator {

        return $this->operSecond;

    }

    /**
     * @param TypesOperator $operatorType
     */
    public function setOperSecond(TypesOperator $operatorType): void {

        $this->operSecond = $operatorType;

    }

    /**
     * @return float|null
     */
    public function getNumSecond(): ?float {

        return $this->numSecond;

    }

    /**
     * @param float $number
     */
    public function setNumSecond(float $number): void {

        $this->numSecond = $number;

    }

    /**
     * @return bool
     */
    public function getEnabled(): bool {

        return $this->enabled;

    }

    /**
     * @param bool $enabled
     */
    public function setEnabled(bool $enabled): void {

        $this->enabled = $enabled;

    }

    /**
     * @return bool
     */
    public function getActive(): bool {

        return $this->active;

    }

    /**
     * @return bool
     */
    public function getDeleted(): bool {

        return $this->deleted;

    }

    /**
     * @param bool $enabled
     */
    public function setDeleted(bool $deleted): void {

        $this->deleted = $deleted;

    }

    /**
     * @return int
     */
    public function getVersion(): int {

        return $this->version;

    }

    /**
     * @param int $version
     */
    public function setVersion(int $version): void {

        $this->version = $version;

    }

}